# Units

## Modules - Concept

- Equivalent to a package in Go
- Contains one or more FSPL source files
- Uniqued by a UUIDv4
- Depends on zero or more other units
- Source files in a module can access functionality of dependencies

## Addressing

When compiling source files, depending on a module, etc. an *address* is used to
refer to a *unit*, which is a module or file. An *addresser* is anything that
addresses a unit, the *addressee*. An addresser can be a module, a user invoking
the compiler, or something else. An address is represented by a string. If the
string ends in `.fspl`, the address refers to an FSPL source file. If not, the
address refers to a module.

If the address begins in a `/`, `./` or `../`, the address is interpreted as an
absolute path beginning from the filesystem root, the current directory of the
addressee, or the directory above that respectively. Otherwise, the unit is
searched for within a set of standard or configured paths.

For example, if the search path is `/usr/include/fspl`, and the address is
`foo`, then the unit will be located at `/usr/include/fspl/foo`. If the address
is `foo/bar`, then the unit will be located at `/usr/include/fspl/foo/bar`. If
there is an additional directory in the search path, such as
`/usr/local/include/fspl`, then the unit will be searched for in each one (in
order) until it is found.

Module UUIDs are not used in addressing.

## Module Structure

Each module is represented by a directory, which contains source files along
with a metadata file called `fspl.mod`. The metadata file is of the form:

```
<file>       -> <UUIDv4> <directive>*
<directive>  -> <depedency>
<dependency> -> "+" <stringLiteral> [<ident>]
<UUIDv4>     -> <stringLiteral>
```

Metadata files only make use of tokens defined in the FSPL lexer, and are
designed to make use of the same parsing and lexing infrastructure used to parse
and tokenize source files. A sample metadata file might look like:

```
'5a8353f8cad84604be6029a2575996bc'
+ 'io'
+ '../io' customIo
```

The UUID is represented as a string, and so are addresses. When depending on a
unit, it may be "nicknamed" by supplying an identifier after the address. This
changes how the unit is referred to within the module.

## Referencing Units

Compiled by itself, an FSPL source file has no access to other units. However,
when compiling a module as a whole, all source files within the module have
access to units depended on by the module's metadata file. Note that no actual
data or code is imported into the module from the units it depends on, because
all methods and functions defined within them are automatically turned into
prototypes. The module must be linked either statically or dynamically to the
unit's object code after compilation. This is why the FSPL compiler outputs
object files by default.

FSPL source files may reference functions or types from dependencies by
prefixing them with a unit name and a double colon (`::`), like this:

```
reader: io::Reader = x
data:   *:Byte     = io::[readAll reader]
```

The name of a unit depends on the associated dependency directive used in the
module metadata file. If a nickname is listed, then that is used as the unit
name. Otherwise, the unit name is the basename of the address, which is
normalized and formatted into a valid identifier by the the following rules:

- If the name contains at least one dot, the last dot and everything after it is
  removed
- All non-alphabetical and non-numeric characters are removed, and any
  alphabetical characters that were directly after them are converted to
  uppercase
- All numeric digits at the start of the string are removed
- The first character is converted to lowercase

For example:

- `100-bottles-of-glue_test`
- `Picture.jpg`
- `Just a straight up sentence`

Would become:

- `bottlesOfGlueTest`
- `picture`
- `justAStraightUpSentence`

If the unit name is still not a valid identifier or is empty, the compiler will
refuse to process the module and it is up to the user to either nickname the
unit, or change the unit's basename to something workable.

The compiler will also refuse to process the module if one or more units end up
with the same unit name. However, a function or a variable may have the 
name as a unit because units are only ever used within the context of their own
special syntax (`::`).

## Future Work

Addresses do not necessarily have to refer to units. They could also refer to
arbitrary blobs to embed into a compiled program, similarly to how Go's embed
system works. There of course would need to be a distinction between depending
on units and embedding data, because someone might want to embed an FSPL source
file. Thus, there would need to be a separate metadata file directive, possibly
starting with an `!` or something like that.

